@@ -9,6 +9,8 @@ module Upgrade.V2 where
 
 import Common.Annex
 import qualified Git
+import qualified Git.Command
+import qualified Git.Ref
 import qualified Annex.Branch
 import Logs.Location
 import Annex.Content
@@ -52,7 +54,7 @@ upgrade = do
 	showProgress
 
 	when e $ do
-		inRepo $ Git.run "rm" [Param "-r", Param "-f", Param "-q", File old]
+		inRepo $ Git.Command.run "rm" [Param "-r", Param "-f", Param "-q", File old]
 		unless bare $ inRepo gitAttributesUnWrite
 	showProgress
 
@@ -86,7 +88,7 @@ logFiles dir = return . filter (".log" `isSuffixOf`)
 
 push :: Annex ()
 push = do
-	origin_master <- Annex.Branch.refExists $ Git.Ref "origin/master"
+	origin_master <- inRepo $ Git.Ref.exists $ Git.Ref "origin/master"
 	origin_gitannex <- Annex.Branch.hasOrigin
 	case (origin_master, origin_gitannex) of
 		(_, True) -> do
@@ -103,7 +105,8 @@ push = do
 			Annex.Branch.update -- just in case
 			showAction "pushing new git-annex branch to origin"
 			showOutput
-			inRepo $ Git.run "push" [Param "origin", Param $ show Annex.Branch.name]
+			inRepo $ Git.Command.run "push"
+				[Param "origin", Param $ show Annex.Branch.name]
 		_ -> do
 			-- no origin exists, so just let the user
 			-- know about the new branch
@@ -126,7 +129,7 @@ gitAttributesUnWrite repo = do
 		c <- readFileStrict attributes
 		liftIO $ viaTmp writeFile attributes $ unlines $
 			filter (`notElem` attrLines) $ lines c
-		Git.run "add" [File attributes] repo
+		Git.Command.run "add" [File attributes] repo
 
 stateDir :: FilePath
 stateDir = addTrailingPathSeparator ".git-annex"
