@@ -6,7 +6,9 @@
  -}
 
 module Utility.Url (
+	URLString,
 	exists,
+	canDownload,
 	download,
 	get
 ) where
@@ -32,24 +34,31 @@ exists url =
 				(2,_,_) -> return True
 				_ -> return False
 
+canDownload :: IO Bool
+canDownload = (||) <$> inPath "wget" <*> inPath "curl"
+
 {- Used to download large files, such as the contents of keys.
+ -
  - Uses wget or curl program for its progress bar. (Wget has a better one,
- - so is preferred.) -}
-download :: URLString -> FilePath -> IO Bool
-download url file = do
+ - so is preferred.) Which program to use is determined at run time; it
+ - would not be appropriate to test at configure time and build support
+ - for only one in.
+ -}
+download :: URLString -> [CommandParam] -> FilePath -> IO Bool
+download url options file = do
 	e <- inPath "wget"
 	if e
 		then
-			boolSystem "wget"
-				[Params "-c -O", File file, File url]
+			go "wget" [Params "-c -O", File file, File url]
 		else
 			-- Uses the -# progress display, because the normal
 			-- one is very confusing when resuming, showing
 			-- the remainder to download as the whole file,
 			-- and not indicating how much percent was
 			-- downloaded before the resume.
-			boolSystem "curl" 
-				[Params "-L -C - -# -o", File file, File url]
+			go "curl" [Params "-L -C - -# -o", File file, File url]
+	where
+		go cmd opts = boolSystem cmd (options++opts)
 
 {- Downloads a small file. -}
 get :: URLString -> IO String
