@@ -2,16 +2,17 @@
 
 import System.Directory
 import Data.List
-import Data.String.Utils
+import Data.Maybe
 import System.Cmd.Utils
 
 import Build.TestConfig
+import Utility.StatFS
 
 tests :: [TestCase]
 tests =
 	[ TestCase "version" getVersion
 	, TestCase "git" $ requireCmd "git" "git --version >/dev/null"
-	, TestCase "git version" checkGitVersion
+	, TestCase "git version" getGitVersion
 	, testCp "cp_a" "-a"
 	, testCp "cp_p" "-p"
 	, testCp "cp_reflink_auto" "--reflink=auto"
@@ -19,8 +20,10 @@ tests =
 	, TestCase "xargs -0" $ requireCmd "xargs_0" "xargs -0 </dev/null"
 	, TestCase "rsync" $ requireCmd "rsync" "rsync --version >/dev/null"
 	, TestCase "curl" $ testCmd "curl" "curl --version >/dev/null"
+	, TestCase "wget" $ testCmd "wget" "wget --version >/dev/null"
 	, TestCase "bup" $ testCmd "bup" "bup --version >/dev/null"
 	, TestCase "gpg" $ testCmd "gpg" "gpg --version >/dev/null"
+	, TestCase "StatFS" testStatFS
 	] ++ shaTestCases [1, 256, 512, 224, 384]
 
 shaTestCases :: [Int] -> [TestCase]
@@ -57,25 +60,16 @@ getVersionString = do
 	where
 		middle = drop 1 . init
 
-{- Checks for a new enough version of git. -}
-checkGitVersion :: Test
-checkGitVersion = do
+getGitVersion :: Test
+getGitVersion = do
 	(_, s) <- pipeFrom "git" ["--version"]
 	let version = last $ words $ head $ lines s
-	if dotted version < dotted need
-		then error $ "git version " ++ version ++ " too old; need " ++ need
-		else return $ Config "gitversion" (StringConfig version)
-	where
-		-- for git-check-attr behavior change
-		need = "1.7.7"
-		dotted = sum . mult 1 . reverse . extend 10 . map readi . split "." 
-		extend n l = l ++ replicate (n - length l) 0
-		mult _ [] = []
-		mult n (x:xs) = (n*x) : mult (n*100) xs
-		readi :: String -> Integer
-		readi s = case reads s of
-			((x,_):_) -> x
-			_ -> 0
+	return $ Config "gitversion" (StringConfig version)
+
+testStatFS :: Test
+testStatFS = do
+	s <- getFileSystemStats "."
+	return $ Config "statfs_sane" $ BoolConfig $ isJust s
 
 {- Set up cabal file with version. -}
 cabalSetup :: IO ()
